@@ -157,7 +157,6 @@ public class PlaybackService extends MediaBrowserServiceCompat{
 
     private int mWidget = 0;
     private boolean mHasAudioFocus = false;
-    private boolean mOnRenderer = false;
     // RemoteControlClient-related
     /**
      * RemoteControlClient is for lock screen playback control.
@@ -229,7 +228,7 @@ public class PlaybackService extends MediaBrowserServiceCompat{
 
     private MedialibraryReceiver mLibraryReceiver = null;
     private void registerMedialibrary(final Runnable action) {
-        if (!Permissions.canReadStorage()) return;
+        if (!Permissions.canReadStorage(this)) return;
         final LocalBroadcastManager lbm = LocalBroadcastManager.getInstance(this);
         if (mLibraryReceiver == null) {
             mLibraryReceiver = new MedialibraryReceiver();
@@ -503,7 +502,7 @@ public class PlaybackService extends MediaBrowserServiceCompat{
                             && playlistManager.switchToVideo()) {
                         hideNotification();
                     } else {
-                        if (!hasRenderer()) showPlayer();
+                        if (!hasRenderer() || !canSwitchToVideo()) showPlayer();
                         showNotification();
                     }
                     break;
@@ -516,10 +515,6 @@ public class PlaybackService extends MediaBrowserServiceCompat{
                     mHandler.removeMessages(SHOW_PROGRESS);
                     if (mWakeLock.isHeld()) mWakeLock.release();
                     break;
-                case MediaPlayer.Event.Stopped:
-                    if (BuildConfig.DEBUG) Log.i(TAG, "MediaPlayer.Event.Stopped");
-                    onPlaybackStopped();
-                    break;
                 case MediaPlayer.Event.EndReached:
                     executeUpdateProgress();
                     break;
@@ -689,11 +684,16 @@ public class PlaybackService extends MediaBrowserServiceCompat{
         }
     }
 
+    private boolean currentMediaHasFlag(int flag) {
+        final MediaWrapper mw = playlistManager.getCurrentMedia();
+        return mw != null && mw.hasFlag(flag);
+    }
+
     public PendingIntent getSessionPendingIntent() {
         if (playlistManager.getPlayer().isVideoPlaying()) { //PIP
             final Intent notificationIntent = new Intent(this, VideoPlayerActivity.class);
             return PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
-        } if (playlistManager.getVideoBackground() || (canSwitchToVideo() && !playlistManager.getCurrentMedia().hasFlag(MediaWrapper.MEDIA_FORCE_AUDIO))) { //resume video playback
+        } if (playlistManager.getVideoBackground() || (canSwitchToVideo() && !currentMediaHasFlag(MediaWrapper.MEDIA_FORCE_AUDIO))) { //resume video playback
             /* Resume VideoPlayerActivity from ACTION_REMOTE_SWITCH_VIDEO intent */
             final Intent notificationIntent = new Intent(Constants.ACTION_REMOTE_SWITCH_VIDEO);
             return PendingIntent.getBroadcast(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
@@ -721,7 +721,6 @@ public class PlaybackService extends MediaBrowserServiceCompat{
     }
 
     public void onNewPlayback(final MediaWrapper mw) {
-        changeAudioFocus(true);
         mMediaSession.setSessionActivity(getSessionPendingIntent());
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
     }
@@ -1128,7 +1127,8 @@ public class PlaybackService extends MediaBrowserServiceCompat{
 
     private String mCurrentWidgetCover = null;
     private void updateWidgetCover() {
-        String newWidgetCover = playlistManager.hasCurrentMedia() ? playlistManager.getCurrentMedia().getArtworkMrl() : null;
+        final MediaWrapper mw = playlistManager.getCurrentMedia();
+        final String newWidgetCover = mw != null ? mw.getArtworkMrl() : null;
         if (!TextUtils.equals(mCurrentWidgetCover, newWidgetCover)) {
             mCurrentWidgetCover = newWidgetCover;
             sendWidgetBroadcast(new Intent(VLCAppWidgetProvider.ACTION_WIDGET_UPDATE_COVER)
@@ -1137,11 +1137,12 @@ public class PlaybackService extends MediaBrowserServiceCompat{
     }
 
     private void updateWidgetPosition(final float pos) {
-        if (mWidget == 0 || isVideoPlaying()) return;
+        final MediaWrapper mw = playlistManager.getCurrentMedia();
+        if (mw == null || mWidget == 0 || isVideoPlaying()) return;
         // no more than one widget mUpdateMeta for each 1/50 of the song
         long timestamp = System.currentTimeMillis();
         if (!playlistManager.hasCurrentMedia()
-                || timestamp - mWidgetPositionTimestamp < playlistManager.getCurrentMedia().getLength() / 50)
+                || timestamp - mWidgetPositionTimestamp < mw.getLength() / 50)
             return;
         mWidgetPositionTimestamp = timestamp;
         sendWidgetBroadcast(new Intent(VLCAppWidgetProvider.ACTION_WIDGET_UPDATE_POSITION)
@@ -1150,8 +1151,7 @@ public class PlaybackService extends MediaBrowserServiceCompat{
 
     private void broadcastMetadata() {
         final MediaWrapper media = playlistManager.getCurrentMedia();
-        if (media == null || isVideoPlaying())
-            return;
+        if (media == null || isVideoPlaying()) return;
         sendBroadcast(new Intent("com.android.music.metachanged")
                 .putExtra("track", media.getTitle())
                 .putExtra("artist", media.getArtist())
@@ -1761,16 +1761,15 @@ public class PlaybackService extends MediaBrowserServiceCompat{
 
     @MainThread
     public boolean hasRenderer() {
-        return mOnRenderer;
+        return playlistManager.getPlayer().getHasRenderer();
     }
 
     @MainThread
-    public int setRenderer(RendererItem item) {
-        final boolean wasOnRenderer = mOnRenderer;
-        mOnRenderer = item != null;
-        if (wasOnRenderer && !mOnRenderer && canSwitchToVideo()) VideoPlayerActivity.startOpened(VLCApplication.getAppContext(),
+    public void setRenderer(RendererItem item) {
+        final boolean wasOnRenderer = hasRenderer();
+        if (wasOnRenderer && !hasRenderer() && canSwitchToVideo()) VideoPlayerActivity.startOpened(VLCApplication.getAppContext(),
                 playlistManager.getCurrentMedia().getUri(), playlistManager.getCurrentIndex());
-        return playlistManager.getPlayer().setRenderer(item);
+        playlistManager.getPlayer().setRenderer(item);
     }
 
     @MainThread
@@ -1871,7 +1870,7 @@ public class PlaybackService extends MediaBrowserServiceCompat{
     @Nullable
     @Override
     public BrowserRoot onGetRoot(@NonNull String clientPackageName, int clientUid, @Nullable Bundle rootHints) {
-        return Permissions.canReadStorage() ? new BrowserRoot(BrowserProvider.ID_ROOT, null) : null;
+        return Permissions.canReadStorage(PlaybackService.this) ? new BrowserRoot(BrowserProvider.ID_ROOT, null) : null;
     }
 
     @Override
