@@ -127,7 +127,6 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
         if (mNetworkRoot)
             vh.binding.setProtocol(getProtocol(media));
         vh.binding.setCover(getIcon(media, mSpecialIcons));
-        vh.setContextMenuListener();
         vh.selectView(media.hasStateFlags(FLAG_SELECTED));
     }
 
@@ -148,6 +147,8 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
 
         public void onClick(View v){}
 
+        public boolean onLongClick(View v){ return false; }
+
         public void onCheckBoxClick(View v){}
 
         public void onMoreClick(View v){}
@@ -156,19 +157,11 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
 
     }
 
-    class MediaViewHolder extends ViewHolder<BrowserItemBinding> implements View.OnLongClickListener, View.OnFocusChangeListener {
+    class MediaViewHolder extends ViewHolder<BrowserItemBinding> implements View.OnFocusChangeListener {
 
         MediaViewHolder(final BrowserItemBinding binding) {
             super(binding);
             binding.setHolder(this);
-            itemView.setOnLongClickListener(new View.OnLongClickListener() {
-                @Override
-                public boolean onLongClick(View v) {
-                    binding.browserCheckbox.toggle();
-                    onCheckBoxClick(binding.browserCheckbox);
-                    return true;
-                }
-            });
             if (AndroidUtil.isMarshMallowOrLater) itemView.setOnContextClickListener(new View.OnContextClickListener() {
                 @Override
                 public boolean onContextClick(View v) {
@@ -178,10 +171,6 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
             });
         }
 
-        void setContextMenuListener() {
-            itemView.setOnLongClickListener(this);
-        }
-
         protected void openStorage() {
             MediaWrapper mw = new MediaWrapper(((Storage) getItem(getLayoutPosition())).getUri());
             mw.setType(MediaWrapper.TYPE_DIR);
@@ -210,9 +199,13 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
                 fragment.onCtxClick(v, position, getDataset().get(position));
         }
 
-        @Override
         public boolean onLongClick(View v) {
             int position = getLayoutPosition();
+            if (getItem(position).getItemType() == TYPE_STORAGE) {
+                binding.browserCheckbox.toggle();
+                onCheckBoxClick(binding.browserCheckbox);
+                return true;
+            }
             return position < getDataset().size() && position >= 0
                     && fragment.onLongClick(v, position, getDataset().get(position));
         }
@@ -237,30 +230,33 @@ public class BaseBrowserAdapter extends SortableAdapter<MediaLibraryItem, BaseBr
     }
 
     public void clear() {
-        if (!isEmpty())
-            update(new ArrayList<MediaLibraryItem>(0));
+        if (!isEmpty()) update(new ArrayList<MediaLibraryItem>(0));
     }
 
     public boolean isEmpty() {
         return Util.isListEmpty(peekLast());
     }
 
+    @MainThread
     public void addItem(MediaLibraryItem item, boolean top) {
         addItem(item, top, -1);
     }
 
-    void addItem(MediaLibraryItem item, boolean top, int positionTo) {
-        int position;
-        List<MediaLibraryItem> list = new ArrayList<>(peekLast());
-        if (positionTo != -1)
-            position = positionTo;
-        else
-            position = top ? mTop : list.size();
+    void addItem(final MediaLibraryItem item, final boolean top, final int positionTo) {
+        VLCApplication.runOnMainThread(new Runnable() {
+            @Override
+            public void run() {
+                int position;
+                final List<MediaLibraryItem> list = new ArrayList<>(peekLast());
+                if (positionTo != -1) position = positionTo;
+                else position = top ? mTop : list.size();
 
-        if (position <= list.size()) {
-            list.add(position, item);
-            update(list);
-        }
+                if (position <= list.size()) {
+                    list.add(position, item);
+                    update(list);
+                }
+            }
+        });
     }
 
     public void setTop (int top) {
