@@ -201,11 +201,9 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
         if (content != null)
             mFoldersContentLists = content;
         @SuppressWarnings("unchecked")
-        final List<MediaLibraryItem> mediaList = (List<MediaLibraryItem>) VLCApplication.getData(KEY_MEDIA_LIST + mMrl);
-        if (!Util.isListEmpty(mediaList))
-            mAdapter.update(mediaList);
-        else
-            mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_REFRESH);
+        final List<MediaLibraryItem> mediaList = mRoot ? null : (List<MediaLibraryItem>) VLCApplication.getData(KEY_MEDIA_LIST + mMrl);
+        if (!Util.isListEmpty(mediaList)) mAdapter.update(mediaList);
+        else mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_REFRESH);
     }
 
     @Override
@@ -245,11 +243,9 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
     public void onSaveInstanceState(Bundle outState){
         outState.putString(KEY_MRL, mMrl);
         outState.putParcelable(KEY_MEDIA, mCurrentMedia);
-        if (mAdapter != null)
-            VLCApplication.storeData(KEY_MEDIA_LIST+mMrl, mAdapter.getAll());
+        if (!mRoot && mAdapter != null) VLCApplication.storeData(KEY_MEDIA_LIST+mMrl, mAdapter.getAll());
         VLCApplication.storeData(KEY_CONTENT_LIST+mMrl, mFoldersContentLists);
-        if (mRecyclerView != null)
-            outState.putInt(KEY_POSITION, mLayoutManager.findFirstCompletelyVisibleItemPosition());
+        if (mRecyclerView != null) outState.putInt(KEY_POSITION, mLayoutManager.findFirstCompletelyVisibleItemPosition());
         super.onSaveInstanceState(outState);
     }
 
@@ -293,7 +289,7 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
         final FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
         final Fragment next = createFragment();
         final Bundle args = new Bundle();
-        VLCApplication.storeData(KEY_MEDIA_LIST+mMrl, mAdapter.getAll());
+        if (!mRoot) VLCApplication.storeData(KEY_MEDIA_LIST+mMrl, mAdapter.getAll());
         VLCApplication.storeData(KEY_CONTENT_LIST+ mMrl, mFoldersContentLists);
         final List<MediaLibraryItem> list = mFoldersContentLists.get(media);
         if (!Util.isListEmpty(list) && !(this instanceof StorageBrowserFragment))
@@ -318,7 +314,12 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
             return;
         }
         final MediaWrapper mediaWrapper = getMediaWrapper(new MediaWrapper(media));
-        mAdapter.addItem(mediaWrapper, false);
+        VLCApplication.runOnMainThread(new Runnable() {
+            @Override
+            public void run() {
+                mAdapter.addItem(mediaWrapper, false);
+            }
+        });
     }
 
     @Override
@@ -381,11 +382,11 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
 
     @Override
     public void refresh() {
+        if (!isResumed()) return;
         if (isSortEnabled()) {
             refreshList = new ArrayList<>();
             refreshing = true;
-        } else
-            mAdapter.clear();
+        } else mAdapter.clear();
         mBrowserHandler.removeCallbacksAndMessages(null);
         mHandler.sendEmptyMessageDelayed(BrowserFragmentHandler.MSG_SHOW_LOADING, 300);
 
@@ -503,7 +504,7 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
 //                    menu.findItem(R.id.directory_view_show_media).setVisible(false);
 //                }
             menu.findItem(R.id.directory_view_play_folder).setVisible(!isEmpty);
-            menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
+            menu.findItem(R.id.directory_view_delete).setVisible(!mRoot && canWrite);
             if (this instanceof NetworkBrowserFragment) {
                 MediaDatabase db = MediaDatabase.getInstance();
                 if (db.networkFavExists(mw.getUri())) {
@@ -565,12 +566,7 @@ public abstract class BaseBrowserFragment extends SortableFragment<BaseBrowserAd
                 }
                 return true;
             case R.id.directory_view_play_folder:
-                List<MediaWrapper> newMediaList = new ArrayList<>();
-                for (MediaLibraryItem mediaItem : mFoldersContentLists.get(mAdapter.get(position))){
-                    if (((MediaWrapper)mediaItem).getType() == MediaWrapper.TYPE_AUDIO || (AndroidUtil.isHoneycombOrLater && ((MediaWrapper)mediaItem).getType() == MediaWrapper.TYPE_VIDEO))
-                        newMediaList.add((MediaWrapper)mediaItem);
-                }
-                MediaUtils.openList(getActivity(), newMediaList, 0);
+                MediaUtils.openMedia(getActivity(), mw);
                 return true;
             case R.id.directory_view_add_playlist:
                 FragmentManager fm = getActivity().getSupportFragmentManager();
